#!/usr/bin/ruby -w
#---
# Excerpted from "Enterprise Integration with Ruby"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_eir for more book information.
#---


require 'rexml/document'
require 'addressbook'
require 'ldap'
require 'thread'
require 'cgi'
require 'webrick'
include WEBrick


# utf8?


class Recipient
  def to_xml
    recipient = REXML::Element.new('recipient')
    recipient.add_attribute('forename', forename)
    recipient.add_attribute('surname', surname)
    address = recipient.add_element('address')
    address.add_element('street').add_text(street)
    address.add_element('postal-code').add_text(postal_code)
    address.add_element('city').add_text(city)
    address.add_element('state').add_text(state)
    recipient.add_element('description').add_text(description)
    recipient
  end
end

class AddressBook
  def to_xml
    addressbook = REXML::Element.new('address-book')
    addressbook.add_attribute('uid', user.uid)
    each_recipient { |r| addressbook.add_element(r.to_xml) }
    addressbook
  end
end



class AddressBookServlet < HTTPServlet::AbstractServlet
  @@instance = nil
  @@instance_creation_mutex = Mutex.new
  
  def self.get_instance(config, *options)
    @@instance_creation_mutex.synchronize do 
      @@instance ||= self.new(config, *options)
    end
  end
  
  def initialize(config, ldap_conn)
    super
    @ldap_conn = ldap_conn
    @ldap_mutex = Mutex.new 
  end
end



class AddressBookServlet
  # Return all entries belonging to a particular address book.
  def do_GET(req, res)
    res['content-type'] = 'text/xml'
    uid = get_uid(req)
    @ldap_mutex.synchronize do 
      begin
        ab = AddressBook.new(@ldap_conn, User.new(uid))
        entries = ab.to_xml
        entries.each_element('recipient') do |r| 
          cn = to_cn(
            r.attributes['forename'],
            r.attributes['surname']
          )
          link = r.add_element('link')
          link.add_text(
            "http://#{req.host}:#{req.port}/ab/#{uid}/#{cn}"
          )
        end
        entries.write(res.body, 0) 
        res.body << "\n"
        res.status = 200
      rescue ResultError
        res.status = 404 
      end
    end
  end
  
  def get_uid(req)
    req.path_info =~ /\/(\d+)$/ ? $1 : nil
  end
  
  def to_cn(forename, surname)
    CGI.escape(forename + " " + surname)
  end
end



class AddressBookServlet
  def do_POST(req, res)
    doc = REXML::Document.new(req.body)    
    if doc.root.name == 'recipient'
      create_recipient(doc, req, res)
    else
      create_address_book(doc, req, res)
    end
  end

  def create_address_book(doc, req, res)
    user = parse_user(doc)
    @ldap_mutex.synchronize do
      AddressBook.create(@ldap_conn, user)
    end
    res.status = 201
    res['Location'] = "/ab/#{user.uid}"
  end
  
  def create_recipient(doc, req, res, update = false)
    uid = get_uid(req)
    recipient = parse_recipient(doc)
    @ldap_mutex.synchronize do
      begin
        ab = AddressBook.new(@ldap_conn, User.new(uid))
        if update
          ab.modify(recipient)
        else
          ab.add(recipient)
        end
        cn = to_cn(recipient.forename, recipient.surname)
        res.status = 201 # Created
        res['Location'] = "/ab/#{uid}/#{cn}"
      rescue ResultError => ex
        @logger.info ex
        res.status = 404
      end
    end    
  end
  
  def parse_user(doc)
    attributes = doc.root.attributes
    User.new(
      attributes['uid'],
      attributes['forename'],
      attributes['surname']
    )
  end

  def parse_recipient(doc)
    attributes = doc.root.attributes
    Recipient.new(
      attributes['forename'],
      attributes['surname'],
      attributes['street'],
      attributes['postal-code'],
      attributes['city'],
      attributes['state'],
      attributes['description']
    )
  end
end



class AddressBookServlet
  def do_PUT(req, res)
    doc = REXML::Document.new(req.body)
    create_recipient(doc, req, res, true)
  end
end



class AddressBookServlet
  def do_DELETE(req, res)
    if req.path_info =~ /\/(\d+)\/(.+)$/
      delete_recipient($1, $2)
    elsif req.path_info =~ /\/(\d+)$/
      delete_address_book($1)
    end
    res.status = 204 # No Content
  end
  
  def delete_recipient(uid, common_name)
    @ldap_mutex.synchronize do
      ab = AddressBook.new(@ldap_conn, User.new(uid))
      cn = CGI.unescape(common_name) 
      surname = cn.split(/.* (\w+)$/)[1]
      forename = cn[0 .. -(surname.size + 1)] 
      ab.remove(Recipient.new(forename, surname))
    end
  end
  
  def delete_address_book(uid)
    @ldap_mutex.synchronize do
      ab = AddressBook.new(@ldap_conn, User.new(uid))
      ab.each_recipient { |r| ab.remove(r) }
      ab.delete
    end
  end
end



connection = Conn.new
connection.set_option(LDAP_OPT_PROTOCOL_VERSION, 3)
connection.bind('cn=root,dc=pragbouquet,dc=com', 'secret')

server = HTTPServer.new(:Port => 4242)
server.mount('/ab', AddressBookServlet, connection) 

%w(INT TERM).each do |signal|
  trap(signal) { server.shutdown } 
end

server.start

